import classnames from 'classnames'
import values from 'lodash.values'
import React, { Component } from 'react'
import { connect } from 'react-redux'

import Button from './Button'
import Html from './Html'
import HighlightSidebar from './HighlightSidebar'
import Loading from './Loading'
import { closeLoading,
  showLoading
} from '../reducers/loading'
import { requestTransactionsContent } from '../reducers/transactions'

const HYPOTHESIS_SEARCH_PATH = 'https://hypothes.is/api/search?url='
const filterTags = ['ClimateFeedback']

class HighlightHtml extends Component {
  constructor () {
    super()
    this.state = {
      highlightsCount: 0,
      intervalsCount: 0,
      highlights: null,
      warning: null
    }
    this.handleSetAnnotations = this._handleSetAnnotations.bind(this)
    this.handleSetAnnotationsCount = this._handleSetAnnotationsCount.bind(this)
    this.handleSetHighlights = this._handleSetHighlights.bind(this)
  }
  _handleSetAnnotations () {
    const { handleSetHighlights } = this
    const { linkUrl,
      showLoading
    } = this.props
    if (!linkUrl) {
      return
    }
    this.setState({
      warning: 'Wait for annotations'
    })
    window.fetch(`${HYPOTHESIS_SEARCH_PATH}${encodeURI(linkUrl)}`)
      .then(result => {
        result.json()
          .then(json => {
            this.handleSetHighlights(json.rows)
          })
      })
  }
  _handleSetAnnotationsCount () {
    const { countDivElement,
      handleSetAnnotations
    } = this
    const { showLoading } = this.props
    const { highlightsCount } = this.state
    if (countDivElement &&
      highlightsCount !== countDivElement.textContent) {
      // update state
      this.setState({
        highlightsCount: countDivElement.textContent
      })
      // just wait a little bit because even if the
      // hyp count has changed, it does not have already now
      // added the span in the html content
      setTimeout(handleSetAnnotations, 100)
    }
  }
  _handleSetHighlights (annotations) {
    const { closeLoading,
      filterTags
    } = this.props
    // scrap hypothesis elements only in our container
    const elements = [
      ...this.divElement.querySelectorAll('hypothesis-highlight')
    ]
    // check that we got several of them
    if (elements.length === 0) {
      this.setState({
        warning: 'Wait for the hypothesis-highlight elements'
      })
      return
    }
    // close loading
    closeLoading()
    // wait until we have the highlights from chrome and the
    // highlights from the apis
    // the list of highlights is normally higher than the highlights one
    // because for one highlighted text, you can have several highlights
    const anchoredAnnotationsById = {}
    const referedAnnotations = []
    // filter the tags
    const filteredAnnotations = annotations && annotations.filter(({tags}) =>
      !filterTags.find(filterTag => !tags.includes(filterTag))
    )
    // then set
    filteredAnnotations.forEach(annotation => {
        if (annotation.references) {
          referedAnnotations.push(annotation)
        } else {
          anchoredAnnotationsById[annotation.id] = annotation
        }
      })
    referedAnnotations.forEach(referedAnnotation => {
      referedAnnotation.references.forEach(reference => {
        const anchoredAnnotation = anchoredAnnotationsById[reference]
        if (anchoredAnnotation) {
          if (typeof anchoredAnnotation.referedAnnotationsById === 'undefined') {
            anchoredAnnotation.referedAnnotationsById = {}
          }
          anchoredAnnotation.referedAnnotationsById[referedAnnotation.id] = referedAnnotation
        }
      })
    })
    // set the highlights by finding the matching elements for each anchored annotation
    const anchoredAnnotations = values(anchoredAnnotationsById)
    const highlights = anchoredAnnotations.map((anchoredAnnotation, index) => {
      const exactText = anchoredAnnotation.target &&
        anchoredAnnotation.target[0] &&
        anchoredAnnotation.target[0].selector &&
        anchoredAnnotation.target[0].selector[3] &&
        anchoredAnnotation.target[0].selector[3].exact
      if (!exactText) {
        return
      }
      const matchedElements = elements.filter(element =>
        exactText.includes(element.textContent))
      if (elements.length === 0) {
        return
      }
      return {
        annotation: anchoredAnnotation,
        exactText,
        elements: matchedElements,
        // we shift of 50px to put the highlight button just over the top
        // baseline of the selection
        top: matchedElements[0].offsetTop - 40
      }
    }).filter(highlight => highlight)
    // update count
    const highlightsCount = this.countDivElement.textContent
    // set state
    this.setState({
      highlightsCount,
      highlights,
      warning: null
    })
  }
  componentDidMount () {
    const { countDivElement,
      handleSetAnnotationsCount
    } = this
    const { showLoading } = this.props
    // add the hyp script
    const scriptElement = document.createElement('script')
    scriptElement.id = 'hypothesis_script'
    scriptElement.type = 'text/javascript'
    scriptElement.async = true
    scriptElement.src = "https://hypothes.is/embed.js"
    this.divElement.appendChild(scriptElement)
    // loading
    showLoading()
    // update warning
    this.setState({
      warning: 'Wait for hypothesis'
    })
    // add the listener on annotation editing
    countDivElement.addEventListener('DOMNodeInserted', handleSetAnnotationsCount)
  }
  componentWillUnmount() {
    const { countDivElement,
      handleSetAnnotationsCount } = this
    // remove
    countDivElement.removeEventListener('DOMNodeInserted', handleSetAnnotationsCount)
    // remove the annotator plugin
    window.annotator && window.annotator.destroy()
    const linkElement = document.querySelector('link[href="https://hypothes.is/app.html"]')
    linkElement.parentElement.removeChild(linkElement)
  }
  render () {
    const { articleId,
      className,
      __html,
      isLoadingActive,
      linkId,
      linkUrl,
      slug
    } = this.props
    const { highlights,
      warning } = this.state
    const isHighlight = highlights && highlights.length > 0
    return (<div
      className={className || 'highlight-html'}
      ref={ _e => this.divElement = _e }
    >
      {
        linkUrl && <link rel='canonical' href={linkUrl} />
      }
      <div className='highlight-html__info'>
        <div
          className='highlight-html__info__count'
          data-hypothesis-annotation-count
          ref={ _e => this.countDivElement = _e }
        />
        <div className='highlight-html__info__loading'>
          {
            isLoadingActive && <Loading />
          }
        </div>
        <Button
            className={classnames(`button highlight-html__info__button`, {
              'button--shadow button--alive': isHighlight && !warning,
              'highlight-html__info__button--disabled': !isHighlight || warning,
              'highlight-html__info__button--warning': warning
            })}
            onClick={() => {
              if (!warning) {
                document.querySelector('.annotator-bucket-indicator.lower')
                        .querySelector('.label')
                        .click()
                return
              }
            }}
          >
            {warning || 'Your claims are downloaded'}
        </Button>
      </div>
      <div className='highlight-html__content'>
        <Html
          __html={__html}
        />
      </div>
      <HighlightSidebar
        articleId={articleId}
        highlights={highlights}
        linkId={linkId}
      />
    </div>)
  }
}

HighlightHtml.defaultProps = {
  filterTags,
  __html: ''
}

function mapStateToProps ({
  loading: {
    isActive
  }
}) {
  return {
    isLoadingActive: isActive
  }
}
export default connect(mapStateToProps, {
  closeLoading,
  showLoading
})(HighlightHtml)
